<?xml version="1.0"?>
<!--
SPDX-FileCopyrightText: 2024 g10 Code GmbH
SPDX-FileContributor: Carl Schwan <carl@carlschwan.eu>
SPDX-License-Identifier: GPL-2.0-or-later
-->
<!DOCTYPE gui SYSTEM "kpartgui.dtd" >
<gui version="2" name="gpgpass" translationDomain="gpgpass">
  <MenuBar>
    <Menu name="file" >
      <Action name="add_entry" />
      <Action name="add_folder" />
      <Separator/>
      <Action name="gpgpass_edit" />
      <Action name="gpgpass_delete" />
      <Separator/>
      <Action name="gpgpass_users" />
      <Action name="gpgpass_users" />
      <Action name="file_quit" />
    </Menu>
    <Menu name="edit" />
  </MenuBar>

 <ToolBar noMerge="1" name="mainToolBar" fullWidth="true" >
  <text>Main Toolbar</text>
  <Action name="add_entry" />
  <Action name="add_folder" />
  <Separator/>
  <Action name="gpgpass_edit" />
  <Action name="gpgpass_delete" />
  <Separator/>
  <Action name="gpgpass_users" />
  <Action name="options_configure" />
 </ToolBar>
</gui>
